--------------------------------------------------------------------------------
-- Weapon Shield Generator
-- Original Carnage Contest Weapon
-- Script by DC, August 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.shieldgen={}
cc.shieldgen.gfx_wpn={}
cc.shieldgen.gen={}
cc.shieldgen.shield0={}
cc.shieldgen.shield1={}
cc.shieldgen.shield2={}
cc.shieldgen.shield3={}

-- Load & Prepare Ressources
for i=0,3,1 do
	cc.shieldgen.gfx_wpn[i]=loadgfx("buildings/shield"..i..".bmp")					-- Weapon Image
	setmidhandle(cc.shieldgen.gfx_wpn[i])
end
cc.shieldgen.sfx_build=loadsfx("buildmetal.ogg")									-- Generator Build Sound
cc.shieldgen.sfx_forcefield=loadsfx("forcefield.ogg")								-- Forcefield Build Sound
cc.shieldgen.sfx_bounce=loadsfx("bounce.wav")										-- Bounce
cc.shieldgen.gfx_icon=loadgfx("buildings/shieldgenerator.png")
setmidhandle(cc.shieldgen.gfx_icon)
cc.shieldgen.gfx_generator=loadgfx("buildings/generator.bmp")
setmidhandle(cc.shieldgen.gfx_generator)

--------------------------------------------------------------------------------
-- Weapon: Shield Generator
--------------------------------------------------------------------------------

cc.shieldgen.id=addweapon("cc.shieldgen","Shield Generator",cc.shieldgen.gfx_icon,0) -- Add Weapon (0 uses)
cc.shieldgen.shields=5																-- Shield Amount

function cc.shieldgen.draw()														-- Draw
	if weapon_shots<6 then
		-- HUD Positioning
		if weapon_shots==0 then
			-- Generator
			hudinfo("Place the generator! Afterwards you will be able to place "..cc.shieldgen.shields.." shields!")
			hudpositioning(pos_build,cc.shieldgen.gfx_generator)
		else
			-- Shield
			hudinfo("Press [Space] to rotate the shield!")
			hudpositioning(pos_build,cc.shieldgen.gfx_wpn[weapon_mode])
			-- HUD ammobar
			if cc.shieldgen.shields-(weapon_shots-1)>0 then
				hudammobar(cc.shieldgen.shields-(weapon_shots-1),cc.shieldgen.shields)
			end
		end
	end
end

function cc.shieldgen.attack(attack)												-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots>0) and (weapon_shots<(cc.shieldgen.shields+1)) and (attack==1) and (weapon_timer==0) then
		-- Rotate
		weapon_timer=15
		weapon_mode=weapon_mode+1
		if weapon_mode>3 then
			weapon_mode=0
		end
	end
	if (weapon_shots<(cc.shieldgen.shields+1)) and (weapon_position==1) then
		weapon_position=0
		-- Use weapon
		if weapon_shots==0 then useweapon(0) end
		weapon_shots=weapon_shots+1
		-- Build
		if weapon_shots==1 then
			-- Build Generator
			createobject(cc.shieldgen.gen.id,weapon_x,weapon_y)
			-- Sound
			playsound(cc.shieldgen.sfx_build)
		else
			-- Build Shields (Parameter = Parent, which is saved in weapon_charge)
			if (getobjecttype(weapon_charge)==cc.shieldgen.gen.id) then
				-- Okay! Generator exists!
				if weapon_mode==0 then createobject(cc.shieldgen.shield0.id,weapon_x,weapon_y,weapon_charge)
				elseif weapon_mode==1 then createobject(cc.shieldgen.shield1.id,weapon_x,weapon_y,weapon_charge) 
				elseif weapon_mode==2 then createobject(cc.shieldgen.shield2.id,weapon_x,weapon_y,weapon_charge) 
				elseif weapon_mode==3 then createobject(cc.shieldgen.shield3.id,weapon_x,weapon_y,weapon_charge) end
				-- FX
				particle(p_muzzle,weapon_x,weapon_y)
				particlesize(1,1)
				particlecolor(150,200,255)
				particlealpha(0.8)
				particlefadealpha(0.01)
				-- Sound
				playsound(cc.shieldgen.sfx_forcefield)
			else
				-- Failure! Generator does not exist!
				particle(p_muzzle,weapon_x,weapon_y)
				particlesize(1,1)
				particlecolor(255,100,150)
				particlealpha(0.8)
				particlefadealpha(0.01)
				-- Sound
				playsound(cc.shieldgen.sfx_forcefield)
				-- End!
				weapon_shots=(cc.shieldgen.shields+1)
			end
		end
		-- End Turn
		if weapon_shots>=(cc.shieldgen.shields+1) then endturn() end
	end
end

--------------------------------------------------------------------------------
-- Object: Generator
--------------------------------------------------------------------------------

cc.shieldgen.gen.id=addobject("cc.shieldgen.gen",cc.shieldgen.gfx_generator)			-- Add Object
function cc.shieldgen.gen.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].health=50
	objects[id].sy=0
	-- Save ID in weapon_charge variable
	weapon_charge=id
end
function cc.shieldgen.gen.draw(id,x,y)
	-- Smoke Particles?
	if objects[id].health<=30 then
		if getframe()%3==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
			particlecolor(25,25,25)
			if getframe()%12==0 then
				particle(p_spark,x-16,y-2+math.random(-4,5))
			end
		end
	else
		if getframe()%9==0 then
			particle(p_smoke,x-7+math.random(-3,3),y-2+math.random(-3,3))
		end		
	end
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1+math.abs(math.sin(id*10+os.clock()*3))*0.1,1+math.abs(math.sin(id*10+os.clock()*3))*0.1)
	setrotation(0)
	-- Draw
	drawimage(cc.shieldgen.gfx_generator,x,y)
end
function cc.shieldgen.gen.update(id,x,y)
	-- Gravity influence on speed
	objects[id].sy=objects[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.abs(objects[id].sy)/15)
	msuby=objects[id].sy/msubt
	for i=1,msubt,1 do
		y=y+msuby
		-- Collision
		if collision(cc.shieldgen.gfx_generator,x,y,1,1,1,-1,id)==1 then
			if (math.abs(objects[id].sy)>0.5) then playsound(cc.shieldgen.sfx_bounce) end
			y=y-msuby
			objects[id].sy=-objects[id].sy*0.3
			msuby=-msuby*0.3
		end
		-- Water
		if y>getwatery()-5 then
			-- Effects
			particle(p_waterhit,x,y)
			playsound(sfx_hitwater1)
			-- Remove Generator
			removeobject(id)
			-- Remove all Shields
			cc.shieldgen.gen.removeshields(id)
			break
		end
	end
	objectposition(id,x,y)
end
function cc.shieldgen.gen.damage(id,damage)
	if objects[id].health>0 then 
		objects[id].health=objects[id].health-damage
		if objects[id].health<=0 then
			local x=getobjectx(id)
			local y=getobjecty(id)
			-- Destroy terrain
			terrainexplosion(x,y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,x,y,math.random(6,9)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,x,y,math.random(6,9)*0.1,grey,grey,grey)
			end
			-- Remove Generator
			removeobject(id)
			-- Remove all Shields
			cc.shieldgen.gen.removeshields(id)
			-- Cause damage
			arealdamage(x,y,100,50)
		end
	end
end
function cc.shieldgen.gen.removeshields(id)
	-- Remove all Shields
	shields=objecttable()
	for i=1,#shields do
		--Is Shield?
		local typ=getobjecttype(shields[i])
		if typ>=cc.shieldgen.shield0.id and typ<=cc.shieldgen.shield3.id then
			if objects[shields[i]].parent==id then
				--Remove FX
				particle(p_muzzle,getobjectx(shields[i]),getobjecty(shields[i]))
				particlesize(1,1)
				particlecolor(255,100,150)
				particlealpha(0.8)
				particlefadealpha(0.01)
				for j=1,3 do
					particle(p_spark,getobjectx(shields[i])+math.random(-5,5),getobjecty(shields[i])+math.random(-5,5))
					particlecolor(255,100,150)
					particlealpha(0.5+math.random()*3.0)
					particlesize(0.8+math.random(2,6)*0.1,0.8+math.random(2,6)*0.1)
				end
				--Remove
				removeobject(shields[i])
			end
		end
	end	
end

--------------------------------------------------------------------------------
-- Objects: Shields
--------------------------------------------------------------------------------

-- Shield Functions
function cc.shieldgen.drawshield(id,t,x,y)
	setcolor(150+math.min(105,objects[id].dmg*5),200-math.min(200,objects[id].dmg*5),255-math.min(255,objects[id].dmg*5))
	setblend(blend_alpha)
	setalpha(0.1+math.abs(math.sin(id*10+os.clock()*5)*0.3))
	setscale(1+math.sin(id*10+os.clock()*5)*0.1,1+math.sin(id*10+os.clock()*5)*0.1)
	setrotation(0)
	drawimage(cc.shieldgen.gfx_wpn[t],x,y)	
end

-- Shield 0
cc.shieldgen.shield0.id=addobject("cc.shieldgen.shield0",cc.shieldgen.gfx_wpn[0])				-- Add Object
function cc.shieldgen.shield0.setup(id,parameter)
	objects[id]={}; objects[id].timer=0; objects[id].dmg=0; objects[id].parent=parameter
end
function cc.shieldgen.shield0.draw(id,x,y)
	cc.shieldgen.drawshield(id,0,x,y)
end
function cc.shieldgen.shield0.update(id,x,y)
	if objects[id].dmg>0 then objects[id].dmg=objects[id].dmg-1 end
end
function cc.shieldgen.shield0.damage(id,damage)
	objects[id].dmg=objects[id].dmg+math.min(100,damage*5)
end

-- Shield 1
cc.shieldgen.shield1.id=addobject("cc.shieldgen.shield1",cc.shieldgen.gfx_wpn[1])				-- Add Object
function cc.shieldgen.shield1.setup(id,parameter)
	objects[id]={}; objects[id].timer=0; objects[id].dmg=0; objects[id].parent=parameter
end
function cc.shieldgen.shield1.draw(id,x,y)
	cc.shieldgen.drawshield(id,1,x,y)
end
function cc.shieldgen.shield1.update(id,x,y)
	if objects[id].dmg>0 then objects[id].dmg=objects[id].dmg-1 end
end
function cc.shieldgen.shield1.damage(id,damage)
	objects[id].dmg=objects[id].dmg+math.min(100,damage*5)
end

-- Shield 2
cc.shieldgen.shield2.id=addobject("cc.shieldgen.shield2",cc.shieldgen.gfx_wpn[2])				-- Add Object
function cc.shieldgen.shield2.setup(id,parameter)
	objects[id]={}; objects[id].timer=0; objects[id].dmg=0; objects[id].parent=parameter
end
function cc.shieldgen.shield2.draw(id,x,y)
	cc.shieldgen.drawshield(id,2,x,y)
end
function cc.shieldgen.shield2.update(id,x,y)
	if objects[id].dmg>0 then objects[id].dmg=objects[id].dmg-1 end
end
function cc.shieldgen.shield2.damage(id,damage)
	objects[id].dmg=objects[id].dmg+math.min(100,damage*5)
end

-- Shield 3
cc.shieldgen.shield3.id=addobject("cc.shieldgen.shield3",cc.shieldgen.gfx_wpn[3])				-- Add Object
function cc.shieldgen.shield3.setup(id,parameter)
	objects[id]={}; objects[id].timer=0; objects[id].dmg=0; objects[id].parent=parameter
end
function cc.shieldgen.shield3.draw(id,x,y)
	cc.shieldgen.drawshield(id,3,x,y)
end
function cc.shieldgen.shield3.update(id,x,y)
	if objects[id].dmg>0 then objects[id].dmg=objects[id].dmg-1 end
end
function cc.shieldgen.shield3.damage(id,damage)
	objects[id].dmg=objects[id].dmg+math.min(100,damage*5)
end