--------------------------------------------------------------------------------
-- Weapon Baby Buggy + Projectile Baby Buggy
-- Original Carnage Contest Weapon
-- Script by DC, September 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.babybuggy={}
cc.babybuggy.babybuggy={}

-- Load & Prepare Ressources
cc.babybuggy.gfx_wpn=loadgfx("weapons/babybuggy.bmp")					-- Weapon Image
setmidhandle(cc.babybuggy.gfx_wpn)
cc.babybuggy.gfx_wpnc=loadgfx("weapons/babybuggyc.bmp")					-- Weapon Collision Image
setmidhandle(cc.babybuggy.gfx_wpnc)
cc.babybuggy.sfx_cry=loadsfx("babycry.ogg")								-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Baby Buggy
--------------------------------------------------------------------------------

cc.babybuggy.id=addweapon("cc.babybuggy","Baby Buggy",cc.babybuggy.gfx_wpn,0,2)	-- Add Weapon (0 uses, first in round 2)

function cc.babybuggy.draw()											-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch the baby buggy (launch it at a downward slope!)")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*15)*5)*getplayerdirection(0))
			drawimage(cc.babybuggy.gfx_wpn,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-5)
		end
	end
end

function cc.babybuggy.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.babybuggy.babybuggy.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)-3
			if collision(cc.babybuggy.gfx_wpnc,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].x=getplayerx(0)
				end
			end
			projectiles[id].px=projectiles[id].x
			projectiles[id].py=projectiles[id].y
			-- Set Speeds
			projectiles[id].sy=0
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			-- Timer
			weapon_timer=50
			-- Cry
			playsound(cc.babybuggy.sfx_cry)
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Baby Buggy
--------------------------------------------------------------------------------

cc.babybuggy.babybuggy.id=addprojectile("cc.babybuggy.babybuggy")		-- Add Projectile

function cc.babybuggy.babybuggy.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(projectiles[id].dir,1)
	-- Calculate projectile rotation
	if projectiles[id].dir==1 then
		setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))+270)
	else
		setscale(1,-1)
		setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))-90)
	end
	-- Draw projectile
	drawimage(cc.babybuggy.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.babybuggy.babybuggy.update(id)								-- Update
	-- Save old position (for rotation calculation)
	if getframe()%5==0 then
		projectiles[id].px=projectiles[id].x
		projectiles[id].py=projectiles[id].y
	end
	-- Timer
	weapon_timer=weapon_timer-1
	-- Move Left/Right
	local explode=0
	projectiles[id].x=projectiles[id].x+projectiles[id].dir*2.5
	if collision(cc.babybuggy.gfx_wpnc,projectiles[id].x,projectiles[id].y)==1 then
		if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
			-- Upwards/Downwards space?
			local doturn=1
			for i=1,3,1 do
				if collision(cc.babybuggy.gfx_wpn,projectiles[id].x,projectiles[id].y+i)==0 then
					doturn=0; projectiles[id].ignore=0 projectiles[id].y=projectiles[id].y+i; break
				elseif terraincollision()==0 and objectcollision()==0 and playercollision()==projectiles[id].ignore then
					doturn=0; projectiles[id].y=projectiles[id].y+i; break
				end
			end
			if doturn==1 then
				explode=1
			end
		end
	else
		projectiles[id].ignore=0
	end
	-- Fall Down
	projectiles[id].sy=projectiles[id].sy+getgravity()
	msubt=math.ceil(math.abs(projectiles[id].sy)/3)
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(cc.babybuggy.gfx_wpnc,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				projectiles[id].sy=0
				projectiles[id].y=projectiles[id].y-msuby
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()-30 then
			stopchannel(0)
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater4)
			-- Free projectile
			freeprojectile(id)
			-- End Turn
			endturn()
			return
		end
	end
	-- Timer / Explode
	if getframesleft()<5 or explode==1 then
		stopchannel(0)
		-- Splatter
		playsound(sfx_splatter1)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,175,75)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,90,1)
		-- Crater
		grey=math.random(0,40)
		terrainalphaimage(gfx_crater200,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end