--------------------------------------------------------------------------------
-- Weapon Bin Laden + Projectile Bin Laden
-- Original Carnage Contest Weapon
-- Script by DC, March 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.binladen={}
cc.binladen.binladen={}

-- Load & Prepare Ressources
cc.binladen.gfx_wpn0=loadgfx("weapons/binladen0.bmp")				-- Weapon Image Frame 0
setmidhandle(cc.binladen.gfx_wpn0)
cc.binladen.gfx_wpn1=loadgfx("weapons/binladen1.bmp")				-- Weapon Image Frame 1
setmidhandle(cc.binladen.gfx_wpn1)
cc.binladen.sfx_attack=loadsfx("binladen.ogg")						-- Attack Sound Loop
cc.binladen.sfx_laugh=loadsfx("osamalaugh.ogg")						-- Laugh Sound

--------------------------------------------------------------------------------
-- Weapon: Bin Laden
--------------------------------------------------------------------------------

cc.binladen.id=addweapon("cc.binladen","Osama bin Laden",cc.binladen.gfx_wpn0,0,2)		-- Add Weapon (0 uses, first in round 2)

function cc.binladen.draw()												-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch Osama bin Laden!")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*15)*5)*getplayerdirection(0))
			drawimage(cc.binladen.gfx_wpn0,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-5)
		end
	end
end

function cc.binladen.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.binladen.binladen.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)-3
			if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].x=getplayerx(0)
				end
			end
			-- Set Speeds
			projectiles[id].sy=0
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			-- Timer
			weapon_timer=50
			-- Laugh
			playsound(cc.binladen.sfx_laugh)
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bin Laden
--------------------------------------------------------------------------------

cc.binladen.binladen.id=addprojectile("cc.binladen.binladen")		-- Add Projectile

function cc.binladen.binladen.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(projectiles[id].dir,1)
	-- Sound
	if channelplaying(0)==0 then playsound(cc.binladen.sfx_attack,0) end
	-- Draw projectile
	if projectiles[id].frame==0 then
		drawimage(cc.binladen.gfx_wpn0,projectiles[id].x,projectiles[id].y-2)
	else
		drawimage(cc.binladen.gfx_wpn1,projectiles[id].x,projectiles[id].y-2)
	end
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	-- Info
	hudinfo("Hit [Space] again to detonate Osama bin Laden!")
end

function cc.binladen.binladen.update(id)							-- Update
	-- Timer
	weapon_timer=weapon_timer-1
	-- Animation
	if collision(colplayer,projectiles[id].x,projectiles[id].y+4)==1 then
		projectiles[id].timer=projectiles[id].timer+1
		if projectiles[id].timer>8 then
			projectiles[id].timer=0
			projectiles[id].frame=1-projectiles[id].frame
		end
	else
		projectiles[id].timer=6
		projectiles[id].frame=1
	end
	-- Move Left/Right
	projectiles[id].x=projectiles[id].x+projectiles[id].dir*0.5
	if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
		if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
			-- Upwards/Downwards space?
			doturn=1
			for i=1,6,1 do
				if (i%2)==1 then offset=(i+1)/2 else offset=-(i/2) end
				if collision(colplayer,projectiles[id].x,projectiles[id].y+offset)==0 then
					doturn=0; projectiles[id].ignore=0 projectiles[id].y=projectiles[id].y+offset; break
				elseif terraincollision()==0 and objectcollision()==0 and playercollision()==projectiles[id].ignore then
					doturn=0; projectiles[id].y=projectiles[id].y+offset; break
				end
			end
			if doturn==1 then
				-- Move Back / Turn
				projectiles[id].x=projectiles[id].x-projectiles[id].dir*0.5
				projectiles[id].dir=projectiles[id].dir*(-1)
			end
		end
	else
		projectiles[id].ignore=0
	end
	-- Fall Down
	projectiles[id].sy=projectiles[id].sy+getgravity()
	msubt=math.ceil(math.abs(projectiles[id].sy)/3)
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				projectiles[id].sy=0
				projectiles[id].y=projectiles[id].y-msuby
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()-30 then
			stopchannel(0)
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater3)
			-- Free projectile
			freeprojectile(id)
			-- End Turn
			endturn()
			return
		end
	end
	-- Timer / Explode
	if getframesleft()<5 or (weapon_timer<=0 and keydown(key_attack)==1) then
		stopchannel(0)
		-- Splatter
		playsound(sfx_splatter2)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,200,100)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,90,1)
		-- Crater
		grey=math.random(0,40)
		terrainalphaimage(gfx_crater200,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end