--------------------------------------------------------------------------------
-- Weapon Frag Grenade + Projectile Frag Grenade + Projectile Cluster
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.fraggrenade={}
cc.fraggrenade.fraggrenade={}
cc.fraggrenade.cluster={}

-- Load & Prepare Ressources
cc.fraggrenade.gfx_wpn=loadgfx("weapons/fraggrenade.bmp")					-- Weapon Image
setmidhandle(cc.fraggrenade.gfx_wpn)
cc.fraggrenade.gfx_cluster=loadgfx("weapons/cluster.bmp")					-- Cluster Image
setmidhandle(cc.fraggrenade.gfx_cluster)
cc.fraggrenade.sfx_attack=loadsfx("throw.ogg")								-- Attack Sound
cc.fraggrenade.sfx_bounce=loadsfx("bounce.wav")								-- Bounce Sound

--------------------------------------------------------------------------------
-- Weapon: Frag Grenade
--------------------------------------------------------------------------------

cc.fraggrenade.id=addweapon("cc.fraggrenade","Frag Grenade",cc.fraggrenade.gfx_wpn,5)	-- Add Weapon (5 uses)

function cc.fraggrenade.draw()													-- Draw
	if weapon_shots<=0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(cc.fraggrenade.gfx_wpn,6,0)
	end
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,100)
	end
	-- HUD Crosshair / HUD Timer
	if weapon_shots==0 then
		hudcrosshair(4,3)
		hudtimer(3,1,10)
	end
end

function cc.fraggrenade.attack(attack)											-- Attack
	if (weapon_shots<=0) then
		-- Charge
		if (attack==1) then
			weapon_charge=weapon_charge+1										-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			playsound(cc.fraggrenade.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.fraggrenade.fraggrenade.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))
			projectiles[id].y=getplayery(0)+3
			-- Initial collision check (avoid throwing into other objects)
			for i=0,10,1 do
				if collision(col5x5,projectiles[id].x+math.sin(math.rad(getplayerrotation(0)))*i,projectiles[id].y-math.cos(math.rad(getplayerrotation(0)))*i)==1 then
					if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
						if (i==0) then
							projectiles[id].x=getplayerx(0)
							projectiles[id].y=getplayery(0)+3
							break
						else
							projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*(i-1)
							projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*(i-1)
							break
						end
					end
				end
			end
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			-- Set timer
			projectiles[id].timer=weapon_timer*50
			-- Effects
			recoil(2)
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Frag Grenade
--------------------------------------------------------------------------------

cc.fraggrenade.fraggrenade.id=addprojectile("cc.fraggrenade.fraggrenade")	-- Add Projectile

function cc.fraggrenade.fraggrenade.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.fraggrenade.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.fraggrenade.fraggrenade.update(id)								-- Update
	-- Gravity influence on speed
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(cc.fraggrenade.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.3
				msubx=-msubx*0.3
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(cc.fraggrenade.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
				projectiles[id].sx=projectiles[id].sx*0.9
				msubx=msubx*0.9
			end
		else
			projectiles[id].ignore=0
		end		
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Timer -> Explode
	projectiles[id].timer=projectiles[id].timer-1
	if projectiles[id].timer<=0 then
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,60,15)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,15,1)
		-- Crater
		grey=math.random(0,40)
		terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		-- Free projectile
		freeprojectile(id)
		-- Spawn Clusters
		math.randomseed(projectiles[id].x*3+projectiles[id].y)
		for i=-2,2,1 do
			pid=createprojectile(cc.fraggrenade.cluster.id)
			projectiles[pid]={}
			projectiles[pid].x=projectiles[id].x+math.random(-20,20)*0.1
			projectiles[pid].y=projectiles[id].y+math.random(-20,20)*0.1
			projectiles[pid].sx=math.random(-10,10)*0.1
			projectiles[pid].sy=math.random(-10,10)*0.1
			if i==0 then projectiles[pid].follow=1 end
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end

--------------------------------------------------------------------------------
-- Projectile: Cluster
--------------------------------------------------------------------------------

cc.fraggrenade.cluster.id=addprojectile("cc.fraggrenade.cluster")		-- Add Projectile

function cc.fraggrenade.cluster.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.fraggrenade.gfx_cluster,projectiles[id].x,projectiles[id].y)
end

function cc.fraggrenade.cluster.update(id)								-- Update
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Particle Tail
	particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
	particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
	particlefadealpha(0.05)
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.15
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
			-- Cause damage
			arealdamage(projectiles[id].x,projectiles[id].y,50,12)
			-- Destroy terrain
			terrainexplosion(projectiles[id].x,projectiles[id].y,12,1)
			-- Crater
			grey=math.random(0,40)
			terrainalphaimage(gfx_crater75,projectiles[id].x,projectiles[id].y,math.random(4,6)*0.1,grey,grey,grey)
			-- Free projectile
			freeprojectile(id)
			break
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	if projectiles[id].follow==1 then
		scroll(projectiles[id].x,projectiles[id].y)
	end
end