--------------------------------------------------------------------------------
-- Weapon Glue Grenade + Projectile Glue Grenade
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.gluegrenade={}
cc.gluegrenade.gluegrenade={}

-- Load & Prepare Ressources
cc.gluegrenade.gfx_wpn=loadgfx("weapons/gluegrenade.bmp")				-- Weapon Image
setmidhandle(cc.gluegrenade.gfx_wpn)
cc.gluegrenade.sfx_attack=loadsfx("throw.ogg")							-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Glue Grenade
--------------------------------------------------------------------------------

cc.gluegrenade.id=addweapon("cc.gluegrenade","Glue Grenade",cc.gluegrenade.gfx_wpn,1)	-- Add Weapon (1 use)

function cc.gluegrenade.draw()											-- Draw
	if weapon_shots<=0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(cc.gluegrenade.gfx_wpn,6,0)
	end
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,100)
	end
	-- HUD Crosshair / HUD Timer
	if weapon_shots==0 then
		hudcrosshair(4,3)
		hudtimer(3,1,10)
	end
end

function cc.gluegrenade.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Charge
		if (attack==1) then
			weapon_charge=weapon_charge+1							-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			playsound(cc.gluegrenade.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.gluegrenade.gluegrenade.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))
			projectiles[id].y=getplayery(0)+3
			-- Initial collision check (avoid throwing into other objects)
			for i=0,10,1 do
				if collision(col5x5,projectiles[id].x+math.sin(math.rad(getplayerrotation(0)))*i,projectiles[id].y-math.cos(math.rad(getplayerrotation(0)))*i)==1 then
					if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
						if (i==0) then
							projectiles[id].x=getplayerx(0)
							projectiles[id].y=getplayery(0)+3
							break
						else
							projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*(i-1)
							projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*(i-1)
							break
						end
					end
				end
			end
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			-- Set timer
			projectiles[id].timer=weapon_timer*50
			-- Collided
			projectiles[id].collided=0
			-- Effects
			recoil(2)
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: gluegrenade
--------------------------------------------------------------------------------

cc.gluegrenade.gluegrenade.id=addprojectile("cc.gluegrenade.gluegrenade")	-- Add Projectile

function cc.gluegrenade.gluegrenade.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.gluegrenade.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.gluegrenade.gluegrenade.update(id)								-- Update
	-- Gravity influence on speed
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	if projectiles[id].collided==0 then
		msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
		msubx=projectiles[id].sx/msubt
		msuby=projectiles[id].sy/msubt
		for i=1,msubt,1 do
			-- Move X
			projectiles[id].x=projectiles[id].x+msubx
			if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].collided=1
					playsound(sfx_splatter5)
					terrainalphaimage(gfx_blood25,projectiles[id].x,projectiles[id].y,0.7,200,200,0)
					break
				end
			else
				projectiles[id].ignore=0
			end
			-- Move Y
			projectiles[id].y=projectiles[id].y+msuby
			if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].collided=1
					playsound(sfx_splatter5)
					terrainalphaimage(gfx_blood25,projectiles[id].x,projectiles[id].y,0.7,200,200,0)
					break
				end
			else
				projectiles[id].ignore=0
			end		
			-- Water
			if (projectiles[id].y)>getwatery()+5 then
				-- Effects
				particle(p_waterhit,projectiles[id].x,projectiles[id].y)
				playsound(sfx_hitwater1)
				-- Free projectile
				freeprojectile(id)
				break
			end
		end
	end
	-- Timer -> Explode
	projectiles[id].timer=projectiles[id].timer-1
	if projectiles[id].timer<=0 then
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,100,50)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,30,1)
		-- Crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		-- Free projectile
		freeprojectile(id)
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end