--------------------------------------------------------------------------------
-- Weapon Heavy Airstrike + Projectile Cluster
-- Original Carnage Contest Weapon
-- Script by DC, September 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.heavyairstrike={}
cc.heavyairstrike.cluster={}

-- Load & Prepare Ressources
cc.heavyairstrike.gfx_wpn=loadgfx("weapons/rc.bmp")								-- Weapon Image
setmidhandle(cc.heavyairstrike.gfx_wpn)
cc.heavyairstrike.gfx_icon=loadgfx("weapons/heavyairstrikeicon.png")			-- Weapon Icon
setmidhandle(cc.heavyairstrike.gfx_icon)
cc.heavyairstrike.gfx_pro=loadgfx("weapons/cluster.bmp")						-- Projectile Image
setmidhandle(cc.heavyairstrike.gfx_pro)
cc.heavyairstrike.sfx_attack=loadsfx("airstrike.ogg")							-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Heavy Airstrike
--------------------------------------------------------------------------------

cc.heavyairstrike.id=addweapon("cc.heavyairstrike","Heavy Air Strike",cc.heavyairstrike.gfx_icon,0,3)	-- Add Weapon (0 uses, first in round 3)

function cc.heavyairstrike.draw()												-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.heavyairstrike.gfx_wpn,7,0)
	-- HUD Positioning
	if weapon_shots==0 then
		hudpositioning(pos_invisible)
	end
end

function cc.heavyairstrike.attack(attack)										-- Attack
	if (weapon_shots<=0) and (weapon_position==1) then
		-- No more weapon switching!
		useweapon(0)
		playsound(cc.heavyairstrike.sfx_attack)
		weapon_shots=weapon_shots+1
		-- Attack
		for i=0,2,1 do
			for j=-2,2,1 do
				pid=createprojectile(cc.heavyairstrike.cluster.id)
				projectiles[pid]={}
				projectiles[pid].x=weapon_x+(j*15)
				projectiles[pid].y=-500-(i*500)
				projectiles[pid].sx=0
				projectiles[pid].sy=5.0
				if i==0 and j==0 then projectiles[pid].follow=1 end
			end
		end
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Projectile: Cluster
--------------------------------------------------------------------------------

cc.heavyairstrike.cluster.id=addprojectile("cc.heavyairstrike.cluster")	-- Add Projectile

function cc.heavyairstrike.cluster.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.heavyairstrike.gfx_pro,projectiles[id].x,projectiles[id].y)
end

function cc.heavyairstrike.cluster.update(id)							-- Update
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Particle Tail
	particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
	particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
	particlefadealpha(0.05)
	-- Gravity influence on speed
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
			-- Cause damage
			arealdamage(projectiles[id].x,projectiles[id].y,70,30)
			-- Destroy terrain
			terrainexplosion(projectiles[id].x,projectiles[id].y,30,1)
			-- Crater
			grey=math.random(0,40)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(5,7)*0.1,grey,grey,grey)
			else
				terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(5,7)*0.1,grey,grey,grey)
			end
			-- Free projectile
			freeprojectile(id)
			break
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	if projectiles[id].follow==1 then
		scroll(projectiles[id].x,projectiles[id].y)
	end
end