--------------------------------------------------------------------------------
-- Weapon Hitler + Projectile Hitler
-- Original Carnage Contest Weapon
-- Script by DC, March 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.hitler={}
cc.hitler.hitler={}

-- Load & Prepare Ressources
cc.hitler.gfx_wpn0=loadgfx("weapons/hitler0.bmp")				-- Weapon Image Frame 0
setmidhandle(cc.hitler.gfx_wpn0)
cc.hitler.gfx_wpn1=loadgfx("weapons/hitler1.bmp")				-- Weapon Image Frame 1
setmidhandle(cc.hitler.gfx_wpn1)
cc.hitler.sfx_attack=loadsfx("hitler.ogg")						-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Hitler
--------------------------------------------------------------------------------

cc.hitler.id=addweapon("cc.hitler","Hitler",cc.hitler.gfx_wpn0,0,2)		-- Add Weapon (0 uses, first in round 2)

function cc.hitler.draw()												-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch Hitler!")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*15)*5)*getplayerdirection(0))
			drawimage(cc.hitler.gfx_wpn0,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-5)
		end
	end
end

function cc.hitler.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.hitler.hitler.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)-3
			if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].x=getplayerx(0)
				end
			end
			-- Set Speeds
			projectiles[id].sy=0
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			-- Timer
			weapon_timer=50
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Hitler
--------------------------------------------------------------------------------

cc.hitler.hitler.id=addprojectile("cc.hitler.hitler")		-- Add Projectile

function cc.hitler.hitler.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(projectiles[id].dir,1)
	-- Draw projectile
	if projectiles[id].frame==0 then
		drawimage(cc.hitler.gfx_wpn0,projectiles[id].x,projectiles[id].y-2)
	else
		drawimage(cc.hitler.gfx_wpn1,projectiles[id].x,projectiles[id].y-2)
	end
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	-- Info
	hudinfo("Hit [Space] again to detonate Hitler!")
end

function cc.hitler.hitler.update(id)							-- Update
	-- Timer
	weapon_timer=weapon_timer-1
	-- Animation
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>19 then
		projectiles[id].timer=0
		projectiles[id].frame=1-projectiles[id].frame
		if projectiles[id].frame==1 then
			-- Yell "Heil!"
			playsound(cc.hitler.sfx_attack)
			-- Punch Players
			if collision(col20x20,projectiles[id].x+projectiles[id].dir*11,projectiles[id].y-5,0,1,0)==1 then
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					playerpush(playercollision(),projectiles[id].dir*3,-10)
					playerdamage(playercollision(),10)
					playsound(sfx_splatter1)
					particle(p_ring,getplayerx(playercollision()),getplayery(playercollision()))
					particlecolor(255,0,0)
					blood(getplayerx(playercollision()),getplayery(playercollision()))
				end
			end
		end
	end
	-- Move Left/Right
	projectiles[id].x=projectiles[id].x+projectiles[id].dir*0.5
	if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
		if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
			-- Upwards/Downwards space?
			doturn=1
			for i=1,6,1 do
				if (i%2)==1 then offset=(i+1)/2 else offset=-(i/2) end
				if collision(colplayer,projectiles[id].x,projectiles[id].y+offset)==0 then
					doturn=0; projectiles[id].ignore=0 projectiles[id].y=projectiles[id].y+offset; break
				elseif terraincollision()==0 and objectcollision()==0 and playercollision()==projectiles[id].ignore then
					doturn=0; projectiles[id].y=projectiles[id].y+offset; break
				end
			end
			if doturn==1 then
				-- Move Back / Turn
				projectiles[id].x=projectiles[id].x-projectiles[id].dir*0.5
				if terraincollision()==1 or objectcollision()>0 then projectiles[id].dir=projectiles[id].dir*(-1) end
			end
		end
	else
		projectiles[id].ignore=0
	end
	-- Fall Down
	projectiles[id].sy=projectiles[id].sy+getgravity()
	msubt=math.ceil(math.abs(projectiles[id].sy)/3)
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(colplayer,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				projectiles[id].sy=0
				projectiles[id].y=projectiles[id].y-msuby
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()-30 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater3)
			-- Free projectile
			freeprojectile(id)
			-- End Turn
			endturn()
			return
		end
	end
	-- Timer / Explode
	if getframesleft()<5 or (weapon_timer<=0 and keydown(key_attack)==1) then
		-- Splatter
		playsound(sfx_splatter2)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,125,65)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,55,1)
		-- Crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater150,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater175,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end