--------------------------------------------------------------------------------
-- Weapon Ice Grenade + Projectile Ice Grenade
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.icegrenade={}
cc.icegrenade.icegrenade={}

-- Load & Prepare Ressources
cc.icegrenade.gfx_wpn=loadgfx("weapons/icegrenade.bmp")					-- Weapon Image
setmidhandle(cc.icegrenade.gfx_wpn)
cc.icegrenade.gfx_ice=loadgfx("weapons/ice.bmp")						-- Impact Ice
cc.icegrenade.sfx_attack=loadsfx("throw.ogg")							-- Attack Sound
cc.icegrenade.sfx_bounce=loadsfx("bounce.wav")							-- Bounce Sound
cc.icegrenade.sfx_freeze=loadsfx("freeze.wav")							-- Freeze Sound

--------------------------------------------------------------------------------
-- Weapon: Ice Grenade
--------------------------------------------------------------------------------

cc.icegrenade.id=addweapon("cc.icegrenade","Ice Grenade",cc.icegrenade.gfx_wpn,1,2)	-- Add Weapon (1 use, first in round 2)

function cc.icegrenade.draw()													-- Draw
	if weapon_shots<=0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(cc.icegrenade.gfx_wpn,6,0)
	end
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,100)
	end
	-- HUD Crosshair / HUD Timer
	if weapon_shots==0 then
		hudcrosshair(4,3)
		hudtimer(3,1,10)
	end
end

function cc.icegrenade.attack(attack)											-- Attack
	if (weapon_shots<=0) then
		-- Charge
		if (attack==1) then
			weapon_charge=weapon_charge+1										-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			playsound(cc.icegrenade.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.icegrenade.icegrenade.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))
			projectiles[id].y=getplayery(0)+3
			-- Initial collision check (avoid throwing into other objects)
			for i=0,10,1 do
				if collision(col5x5,projectiles[id].x+math.sin(math.rad(getplayerrotation(0)))*i,projectiles[id].y-math.cos(math.rad(getplayerrotation(0)))*i)==1 then
					if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
						if (i==0) then
							projectiles[id].x=getplayerx(0)
							projectiles[id].y=getplayery(0)+3
							break
						else
							projectiles[id].x=getplayerx(0)+(4*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*(i-1)
							projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*(i-1)
							break
						end
					end
				end
			end
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			-- Set timer
			projectiles[id].timer=weapon_timer*50
			-- Effects
			recoil(2)
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Ice Grenade
--------------------------------------------------------------------------------

cc.icegrenade.icegrenade.id=addprojectile("cc.icegrenade.icegrenade")	-- Add Projectile

function cc.icegrenade.icegrenade.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.icegrenade.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.icegrenade.icegrenade.update(id)							-- Update
	-- Particle Tail
	particle(p_smoke,projectiles[id].x,projectiles[id].y)
	particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
	particlecolor(255,255,255)
	-- Gravity influence on speed
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(cc.icegrenade.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.3
				msubx=-msubx*0.3
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(cc.icegrenade.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
				projectiles[id].sx=projectiles[id].sx*0.9
				msubx=msubx*0.9
			end
		else
			projectiles[id].ignore=0
		end		
		-- Water
		if (projectiles[id].y)>getwatery()-45 then
			projectiles[id].timer=0
			break
		end
	end
	-- Timer -> Explode
	projectiles[id].timer=projectiles[id].timer-1
	if projectiles[id].timer<=0 then
		playsound(cc.icegrenade.sfx_freeze)
		-- Crater
		blue=math.random(200,255)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
		else
			terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
		end
		-- Ice
		terrainimage(cc.icegrenade.gfx_ice,projectiles[id].x,projectiles[id].y)
		-- Freeze Players
		players=playertable()
		for i=1,#players,1 do
			if getplayerhealth(players[i])>0 then
				if math.sqrt((getplayerx(players[i])-projectiles[id].x)^2+(getplayery(players[i])-projectiles[id].y)^2)<=45 then
					playerstate(players[i],state_frozen,1)
				end
			end
		end
		-- Particles
		particle(p_muzzle,projectiles[id].x,projectiles[id].y)
		particlecolor(255,255,255)
		for angle=0,350,10 do
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			particlespeed(math.sin(math.rad(angle))*2,-math.cos(math.rad(angle))*2)
			particlecolor(255,255,255)
		end
		-- Free projectile
		freeprojectile(id)
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end