--------------------------------------------------------------------------------
-- Weapon Laser + Projectile Ray
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.laser={}
cc.laser.ray={}

-- Load & Prepare Ressources
cc.laser.gfx_wpn=loadgfx("weapons/laser.bmp")					-- Weapon Image
setmidhandle(cc.laser.gfx_wpn)
cc.laser.gfx_pro=loadgfx("weapons/lasershot.png")				-- Projectile Image
setmidhandle(cc.laser.gfx_pro)
cc.laser.sfx_attack=loadsfx("laser.wav")						-- Attack Sound
cc.laser.sfx_impact=loadsfx("laserimpact.wav")					-- Impact Sound

--------------------------------------------------------------------------------
-- Weapon: Laser
--------------------------------------------------------------------------------

cc.laser.id=addweapon("cc.laser","Laser",cc.laser.gfx_wpn,3)	-- Add Weapon (3 uses)
cc.laser.ammo=5													-- 5 Rays

function cc.laser.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.laser.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.laser.ammo-weapon_shots>0 then
		hudammobar(cc.laser.ammo-weapon_shots,cc.laser.ammo)
	end
	-- HUD Crosshair
	if cc.laser.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.laser.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.laser.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Reset Timer
		weapon_timer=15
		-- Attack
		playsound(cc.laser.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.laser.ray.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*15.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*15.0
		-- Timer
		projectiles[id].timer=0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.laser.ray.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlecolor(255,0,0)
		particlefadealpha(0.01)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		if (weapon_shots>=cc.laser.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Ray
--------------------------------------------------------------------------------

cc.laser.ray.id=addprojectile("cc.laser.ray")			-- Add Projectile

function cc.laser.ray.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,0,0)
	setscale(0.5,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.laser.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.laser.ray.update(id)						-- Update
	-- Move
	cc.laser.ray.move(id)
end

function cc.laser.ray.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>500 then
		-- Free projectile
		freeprojectile(id)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					playerpush(playercollision(),projectiles[id].sx/10.0,projectiles[id].sy/10.0)
					playerdamage(playercollision(),6)
					blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),6)
				end
				-- Destroy terrain
				terraincircle(projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21,7,0x00000000)
				-- Effects
				playsound(cc.laser.sfx_impact)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlecolor(255,0,0)
				particlefadealpha(0.01)
				for i=1,3 do
					particle(p_spark,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
					particlecolor(255,0,0)
					particlefadealpha(0.004)
				end
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			playsound(cc.laser.sfx_impact)
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end