--------------------------------------------------------------------------------
-- Weapon Mine + Projectile Mine
-- Original Carnage Contest Weapon
-- Script by DC, March 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.mine={}
cc.mine.mine={}

-- Load & Prepare Ressources
cc.mine.gfx_wpn=loadgfx("weapons/mine.bmp")						-- Weapon Image
setmidhandle(cc.mine.gfx_wpn)
cc.mine.sfx_attack=loadsfx("throw.ogg")							-- Attack Sound
cc.mine.sfx_bounce=loadsfx("bounce.wav")						-- Bounce Sound

--------------------------------------------------------------------------------
-- Weapon: mine
--------------------------------------------------------------------------------

cc.mine.id=addweapon("cc.mine","Mine",cc.mine.gfx_wpn,1)				-- Add Weapon (1 use)

function cc.mine.draw()													-- Draw
	if getplayeraction(0)==0 and weapon_shots<=0 then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setscale(1,1)
		setrotation(0)
		drawimage(cc.mine.gfx_wpn,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)+2)
	end
end

function cc.mine.attack(attack)											-- Attack
	if (weapon_shots<=0) then
		if attack==1 then
			-- No more weapon switching!
			useweapon(0)
			playsound(cc.mine.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.mine.mine.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*7
			projectiles[id].y=getplayery(0)+2
			-- Set speed of projectile
			projectiles[id].sx=getplayerdirection(0)*0.5
			projectiles[id].sy=-1
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: mine
--------------------------------------------------------------------------------

cc.mine.mine.id=addprojectile("cc.mine.mine")	-- Add Projectile

function cc.mine.mine.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	-- Draw projectile
	drawimage(cc.mine.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.mine.mine.update(id)					-- Update
	-- Gravity influence on speed + decrease x speed
	projectiles[id].sx=projectiles[id].sx*0.95
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(cc.mine.gfx_wpn,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(cc.mine.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.05
				msubx=-msubx*0.05
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(cc.mine.gfx_wpn,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(cc.mine.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
				-- Remove / Exchange with real mine
				if (math.abs(projectiles[id].sy)<0.3) then
					createobject(o_mine,projectiles[id].x,projectiles[id].y)
					freeprojectile(id)
				end
			end
		else
			projectiles[id].ignore=0
		end		
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end
