--------------------------------------------------------------------------------
-- Weapon Missile Battery + Projectile Missile
-- Original Carnage Contest Weapon
-- Script by DC, September 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.missilebat={}
cc.missilebat.missile={}

-- Load & Prepare Ressources
cc.missilebat.gfx_wpn=loadgfx("weapons/missilebattery.bmp")				-- Weapon Image
setmidhandle(cc.missilebat.gfx_wpn)
cc.missilebat.gfx_pro=loadgfx("weapons/missile.bmp")					-- Projectile Image
setmidhandle(cc.missilebat.gfx_pro)
cc.missilebat.sfx_attack=loadsfx("rocketrelease.wav")					-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Missile Battery
--------------------------------------------------------------------------------

cc.missilebat.id=addweapon("cc.missilebat","Missile Battery",cc.missilebat.gfx_wpn,0,3)	-- Add Weapon (0 uses, first in round 3)
cc.missilebat.ammo=9													-- 9 Missiles

function cc.missilebat.draw()											-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(160,200,160)
	drawinhand(cc.missilebat.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.missilebat.ammo-weapon_shots>0 then
		hudammobar(cc.missilebat.ammo-weapon_shots,cc.missilebat.ammo)
	end
	-- HUD Crosshair
	if cc.missilebat.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.missilebat.attack(attack)									-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots<cc.missilebat.ammo) and (attack==1) and (weapon_timer<=0) then
		-- No more weapon switching!
		useweapon(0)
		playsound(cc.missilebat.sfx_attack)
		weapon_shots=weapon_shots+1
		weapon_timer=15
		id=createprojectile(cc.missilebat.missile.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(6*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*10.0
		projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*10.0
		-- Initial movement
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*20
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*20
		projectiles[id].x=projectiles[id].x-projectiles[id].sx
		projectiles[id].y=projectiles[id].y-projectiles[id].sy
		cc.missilebat.missile.move(id,0)
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*10.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*10.0
		-- Effects
		recoil(5)
		-- End Turn
		if weapon_shots>=cc.missilebat.ammo then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: missile
--------------------------------------------------------------------------------

cc.missilebat.missile.id=addprojectile("cc.missilebat.missile")		-- Add Projectile

function cc.missilebat.missile.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.missilebat.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.missilebat.missile.update(id)							-- Update
	-- Move
	cc.missilebat.missile.move(id,1)
end

function cc.missilebat.missile.move(id,fx)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Particle Tail
	if (fx==1) then
		particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
		particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
		particlefadealpha(0.01)
		particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(rot))*6,projectiles[id].y+math.cos(math.rad(rot))*6)
		particlefadealpha(0.04)
	end
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.5
	projectiles[id].sy=projectiles[id].sy+getgravity()*1.5
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*3,projectiles[id].y-math.cos(math.rad(rot))*3)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				arealdamage(projectiles[id].x,projectiles[id].y,70,20)
				-- Destroy terrain
				terrainexplosion(projectiles[id].x,projectiles[id].y,20,1)
				-- Crater
				grey=math.random(0,40)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				end
				-- Free projectile
				freeprojectile(id)
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
end