--------------------------------------------------------------------------------
-- Weapon Pac-Man + Projectile Pac-Man
-- Original Carnage Contest Weapon
-- Script by DC, May 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.pacman={}
cc.pacman.pacman={}

-- Load & Prepare Ressources
cc.pacman.gfx_wpn0=loadgfx("weapons/pacman0.png")				-- Weapon Image Frame 0
setmidhandle(cc.pacman.gfx_wpn0)
cc.pacman.gfx_wpn1=loadgfx("weapons/pacman1.png")				-- Weapon Image Frame 1
setmidhandle(cc.pacman.gfx_wpn1)
cc.pacman.sfx_attack=loadsfx("pacman.ogg")						-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: Pac-Man
--------------------------------------------------------------------------------

cc.pacman.id=addweapon("cc.pacman","Pac-Man",cc.pacman.gfx_wpn0,0,3)	-- Add Weapon (0 uses, first in round 3)

function cc.pacman.draw()												-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch Pac-Man!")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*15)*5)*getplayerdirection(0))
			drawimage(cc.pacman.gfx_wpn0,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-5)
			-- HUD Crosshair
			hudcrosshair(6,3)
		end
	end
end

function cc.pacman.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.pacman.pacman.id)
			projectiles[id]={}
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)-3
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			-- Timer
			weapon_timer=50
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Pac-Man
--------------------------------------------------------------------------------

cc.pacman.pacman.id=addprojectile("cc.pacman.pacman")			-- Add Projectile

function cc.pacman.pacman.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	if projectiles[id].dir==-1 then
		setrotation(-180)
	elseif projectiles[id].dir==1 then
		setrotation(0)
	elseif projectiles[id].dir==-2 then
		setrotation(-90)
	elseif projectiles[id].dir==2 then
		setrotation(90)
	end
	-- Draw projectile
	if projectiles[id].frame==0 then
		drawimage(cc.pacman.gfx_wpn0,projectiles[id].x,projectiles[id].y-2)
	else
		drawimage(cc.pacman.gfx_wpn1,projectiles[id].x,projectiles[id].y-2)
	end
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	-- Info
	hudinfo("Control Pac-Man with your arrow keys or W/A/S/D!")
end

function cc.pacman.pacman.update(id)							-- Update
	-- Timer
	weapon_timer=weapon_timer-1
	projectiles[id].timer=projectiles[id].timer+1
	-- Animation
	if projectiles[id].timer>6 then
		projectiles[id].timer=0
		projectiles[id].frame=1-projectiles[id].frame
		-- Bite
		if projectiles[id].frame==0 then
			playsound(cc.pacman.sfx_attack,0)
			if collision(cc.pacman.gfx_wpn1,projectiles[id].x,projectiles[id].y,1,0,0)==1 then
				-- Eat Terrain
				terraincircle(projectiles[id].x,projectiles[id].y,17,0x00000000)
			end
		end
	end
	-- Control
	if keydown(key_left)==1 then projectiles[id].dir=-1 end
	if keydown(key_right)==1 then projectiles[id].dir=1 end
	if keydown(key_up)==1 then projectiles[id].dir=-2 end
	if keydown(key_down)==1 then projectiles[id].dir=2 end
	-- Move
	if projectiles[id].dir==-1 then
		projectiles[id].x=projectiles[id].x-1
	elseif projectiles[id].dir==1 then
		projectiles[id].x=projectiles[id].x+1
	elseif projectiles[id].dir==-2 then
		projectiles[id].y=projectiles[id].y-1
	elseif projectiles[id].dir==2 then
		projectiles[id].y=projectiles[id].y+1
	end
	-- Water
	if (projectiles[id].y)>getwatery()-20 then
		-- Effects
		particle(p_waterhit,projectiles[id].x,projectiles[id].y)
		playsound(sfx_hitwater3)
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
		return
	-- Timer / Explode
	elseif getframesleft()<5 or (weapon_timer<=0 and keydown(key_attack)==1) then
		-- Splatter
		playsound(sfx_splatter2)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,60,30)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,50,1)
		-- Crater
		grey=math.random(0,40)
		terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end