--------------------------------------------------------------------------------
-- Weapon Paintgun + Projectile Bullet
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.paintgun={}
cc.paintgun.bullet={}

-- Load & Prepare Ressources
cc.paintgun.gfx_wpn=loadgfx("weapons/paintgun.bmp")						-- Weapon Image
setmidhandle(cc.paintgun.gfx_wpn)
cc.paintgun.gfx_pro=loadgfx("weapons/shot.bmp")							-- Projectile Image
setmidhandle(cc.paintgun.gfx_pro)
cc.paintgun.sfx_attack=loadsfx("paintgun.ogg")							-- Attack Sound

--------------------------------------------------------------------------------
-- Weapon: paintgun
--------------------------------------------------------------------------------

cc.paintgun.id=addweapon("cc.paintgun","Paintgun",cc.paintgun.gfx_wpn)	-- Add Weapon
cc.paintgun.ammo=10														-- 10 Bullets

function cc.paintgun.draw()												-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.paintgun.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.paintgun.ammo-weapon_shots>0 then
		hudammobar(cc.paintgun.ammo-weapon_shots,cc.paintgun.ammo)
	end
	-- HUD Crosshair
	if cc.paintgun.ammo-weapon_shots>0 then
		hudcrosshair(6,3)
	end
end

function cc.paintgun.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.paintgun.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Reset Timer
		weapon_timer=15
		-- Attack
		playsound(cc.paintgun.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.paintgun.bullet.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*15.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*15.0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.paintgun.bullet.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(2)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*16,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*16)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		if (weapon_shots>=cc.paintgun.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Bullet
--------------------------------------------------------------------------------

cc.paintgun.bullet.id=addprojectile("cc.paintgun.bullet")		-- Add Projectile

function cc.paintgun.bullet.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(255,255,0)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.paintgun.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.paintgun.bullet.update(id)							-- Update
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.02
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move
	cc.paintgun.bullet.move(id)
end

function cc.paintgun.bullet.move(id)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					playerdamage(playercollision(),1)
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),1)
				end
				-- Effects
				j=math.random(1,5)
				if j==1 then
					r=255; g=0; b=0
				elseif j==2 then
					r=0; g=255; b=0
				elseif j==3 then
					r=50; g=100; b=255
				elseif j==4 then
					r=200; g=0; b=255
				else
					r=255; g=255; b=0
				end
				j=math.random(1,3)
				x=projectiles[id].x+math.sin(math.rad(rot))*20
				y=projectiles[id].y-math.cos(math.rad(rot))*20
				if j==1 then terrainalphaimage(gfx_blood25,x+math.random(-5,5),y+math.random(-5,5),math.random(7,10)*0.1,r,g,b); playsound(sfx_splatter4) end
				if j==2 then terrainalphaimage(gfx_blood50,x+math.random(-7,7),y+math.random(-7,7),math.random(7,10)*0.1,r,g,b); playsound(sfx_splatter3) end
				if j==3 then terrainalphaimage(gfx_blood75,x+math.random(-10,10),y+math.random(-10,10),math.random(7,10)*0.1,r,g,b); playsound(sfx_splatter2) end
				for k=1,5 do
					particle(p_smoke,x,y)
					particlecolor(r,g,b)
					particlefadealpha(0.005)
					particle(p_blood,x,y)
					particlecolor(r,g,b)
					particlespeed(math.random(-10,10)*0.1,math.random(-10,10)*0.1)
				end
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			if math.random(1,2)==1 then
				playsound(sfx_hitwater2)
			else
				playsound(sfx_hitwater3)
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end