--------------------------------------------------------------------------------
-- Weapon Parasite + Projectile Parasite
-- Original Carnage Contest Weapon
-- Script by DC, September 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.parasite={}
cc.parasite.parasite={}

-- Load & Prepare Ressources
cc.parasite.gfx_wpn0=loadgfx("weapons/parasite0.bmp")					-- Weapon Image Frame 0
setmidhandle(cc.parasite.gfx_wpn0)
cc.parasite.gfx_wpn1=loadgfx("weapons/parasite1.bmp")					-- Weapon Image Frame 1
setmidhandle(cc.parasite.gfx_wpn1)
cc.parasite.sfx_step=loadsfx("parasite.ogg")							-- Sound


--------------------------------------------------------------------------------
-- Weapon: Parasite
--------------------------------------------------------------------------------

cc.parasite.id=addweapon("cc.parasite","Parasite",cc.parasite.gfx_wpn0,0,3)	-- Add Weapon (0 uses, first in round 3)

function cc.parasite.draw()													-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch the parasite!")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*30)*15)*getplayerdirection(0))
			drawimage(cc.parasite.gfx_wpn0,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-1)
		end
	end
end

function cc.parasite.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.parasite.parasite.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)
			if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].x=getplayerx(0)
				end
			end
			projectiles[id].px=projectiles[id].x
			projectiles[id].py=projectiles[id].y
			-- Set Speeds
			projectiles[id].sy=0
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			projectiles[id].infection=0
			-- Timer
			weapon_timer=50
			-- Sound
			--playsound(cc.parasite.sfx_attack)
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Parasite
--------------------------------------------------------------------------------

cc.parasite.parasite.id=addprojectile("cc.parasite.parasite")		-- Add Projectile

function cc.parasite.parasite.draw(id)					-- Draw
	if projectiles[id].infection==0 then
		-- Setup draw mode
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setscale(1,1)
		-- Calculate projectile rotation
		if projectiles[id].dir==1 then
			setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))+270)
		else
			setscale(1,-1)
			setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))-90)
		end
		-- Draw projectile
		if projectiles[id].frame==0 then
			drawimage(cc.parasite.gfx_wpn0,projectiles[id].x,projectiles[id].y-2)
		else
			drawimage(cc.parasite.gfx_wpn1,projectiles[id].x,projectiles[id].y-2)
		end
		-- Draw Arrow if out of Screen
		outofscreenarrow(projectiles[id].x,projectiles[id].y)
		-- Info
		hudinfo("Hit [Space] again to detonate the parasite!")
	end
end

function cc.parasite.parasite.update(id)							-- Update
	if projectiles[id].infection==0 then
		-- Save old position (for rotation calculation)
		if getframesleft()%15==0 then
			projectiles[id].px=projectiles[id].x
			projectiles[id].py=projectiles[id].y
			playsound(cc.parasite.sfx_step)
		end
		-- Timer
		weapon_timer=weapon_timer-1
		-- Animation
		projectiles[id].timer=projectiles[id].timer+1
		if projectiles[id].timer>14 then
			projectiles[id].timer=0
			projectiles[id].frame=1-projectiles[id].frame
		end
		-- Move Left/Right
		projectiles[id].x=projectiles[id].x+projectiles[id].dir*0.5
		if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Infect
				collision(col10x10,projectiles[id].x,projectiles[id].y,0,1,0)
				if playercollision()~=0 and playercollision()~=projectiles[id].ignore then
					projectiles[id].infection=playercollision()
					weapon_timer=0
					playsound(sfx_splatter3)
					playerstate(projectiles[id].infection,state_confused,1)
					-- End Turn
					endturn()
				end
				-- Upwards/Downwards space?
				doturn=1
				for i=1,14,1 do
					if (i%2)==1 then offset=(i+1)/2 else offset=-(i/2) end
					if collision(col3x3,projectiles[id].x,projectiles[id].y+offset)==0 then
						doturn=0; projectiles[id].ignore=0 projectiles[id].y=projectiles[id].y+offset; break
					elseif terraincollision()==0 and objectcollision()>0 and playercollision()==projectiles[id].ignore then
						doturn=0; projectiles[id].y=projectiles[id].y+offset; break
					end
				end
				if doturn==1 then
					-- Move Back / Turn
					projectiles[id].x=projectiles[id].x-projectiles[id].dir*0.5
					projectiles[id].dir=projectiles[id].dir*(-1)
					projectiles[id].px=projectiles[id].x
					projectiles[id].py=projectiles[id].y
				end
			end
		else
			projectiles[id].ignore=0
		end
		if projectiles[id].infection==0 then
			-- Fall Down
			projectiles[id].sy=projectiles[id].sy+getgravity()*2
			msubt=math.ceil(math.abs(projectiles[id].sy)/3)
			msuby=projectiles[id].sy/msubt
			for i=1,msubt,1 do
				projectiles[id].y=projectiles[id].y+msuby
				-- Collision
				if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
					if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
						projectiles[id].sy=0
						projectiles[id].y=projectiles[id].y-msuby
						break
					end
				else
					projectiles[id].ignore=0
				end
				-- Water
				if (projectiles[id].y)>getwatery() then
					playsound(sfx_waterstep)
					-- Free projectile
					freeprojectile(id)
					-- End Turn
					endturn()
					return
				end
			end
			-- Timer / Explode
			if getframesleft()<5 or (weapon_timer<=0 and keydown(key_attack)==1) then
				-- Splatter
				playsound(sfx_splatter2)
				-- Dirt
				terrainalphaimage(gfx_blood50,projectiles[id].x,projectiles[id].y,0.8,255,255,0)
				particle(p_smoke,projectiles[id].x,projectiles[id].y)
				particlealpha(1.0)
				particlecolor(255,255,0)
				particlesize(10,10)
				particlefadealpha(0.01)
				for i=1,5,1 do
					particle(p_blood,projectiles[id].x,projectiles[id].y)
					particlecolor(255,255,0)
				end
				-- Free projectile
				freeprojectile(id)
				-- End Turn
				endturn()
			end
		end
	end
	-- Infection
	if projectiles[id].infection>0 then
		weapon_timer=weapon_timer+1
		if weapon_timer>(50*3) then
			-- Kill Infected
			playerdamage(projectiles[id].infection,10000)
			playsound(sfx_splatter2)
			-- Free projectile
			freeprojectile(id)
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end