--------------------------------------------------------------------------------
-- Weapon RC Missile Launcher + Projectile Missile
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.rcmissile={}
cc.rcmissile.missile={}

-- Load & Prepare Ressources
cc.rcmissile.gfx_wpn=loadgfx("weapons/launcher.bmp")					-- Weapon Image
setmidhandle(cc.rcmissile.gfx_wpn)
cc.rcmissile.gfx_pro=loadgfx("weapons/rcmissile.bmp")					-- Projectile Image
setmidhandle(cc.rcmissile.gfx_pro)
cc.rcmissile.sfx_attack=loadsfx("rocketrelease.wav")					-- Attack Sound
cc.rcmissile.sfx_homing=loadsfx("homing.wav")							-- Homing Sound

--------------------------------------------------------------------------------
-- Weapon: RC Missile Launcher
--------------------------------------------------------------------------------

cc.rcmissile.id=addweapon("cc.rcmissile","RC Missile",cc.rcmissile.gfx_pro,1,2)	-- Add Weapon (1 use, first in round 2)

function cc.rcmissile.draw()											-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,180,120)
	drawinhand(cc.rcmissile.gfx_wpn,6,0)
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,100)
	end
	-- HUD Crosshair
	if weapon_shots==0 then
		hudcrosshair(6,3)
	end
end

function cc.rcmissile.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Charge
		if (attack==1) then
			weapon_charge=weapon_charge+1							-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			-- Disable player control
			playercontrol(0)
			-- Make sure that there is enough round time
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- SFX
			playsound(cc.rcmissile.sfx_attack)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.rcmissile.missile.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(6*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*10.0
			projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*10.0
			-- Set timer
			projectiles[id].timer=15*50
			-- Initial movement
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*20
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*20
			projectiles[id].x=projectiles[id].x-projectiles[id].sx
			projectiles[id].y=projectiles[id].y-projectiles[id].sy
			cc.rcmissile.missile.move(id,0)
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			-- Effects
			recoil(2)
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Missile
--------------------------------------------------------------------------------

cc.rcmissile.missile.id=addprojectile("cc.rcmissile.missile")		-- Add Projectile

function cc.rcmissile.missile.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.rcmissile.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.rcmissile.missile.update(id)							-- Update
	cc.rcmissile.missile.move(id,1)
end

function cc.rcmissile.missile.move(id,fx)
	-- Control by player
	if projectiles[id].timer==(14*50) then
		-- Start Control FX
		playsound(cc.rcmissile.sfx_homing)
		particle(p_ring,projectiles[id].x,projectiles[id].y)
		particlecolor(255,150,0)
	end
	if projectiles[id].timer<=(14*50) then
		rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
		--speed=math.sqrt(math.pow(projectiles[id].sx,2)+math.pow(projectiles[id].sy,2))
		if keydown(key_left)==1 then
			rot=rot-3
		elseif keydown(key_right)==1 then
			rot=rot+3
		end
		projectiles[id].sx=math.sin(math.rad(rot))*6.0
		projectiles[id].sy=-math.cos(math.rad(rot))*6.0
	else
		rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	end
	-- Particle Tail
	if (fx==1) then
		particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
		particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
		particlefadealpha(0.01)
		if projectiles[id].timer<=(14*50) then
			particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(rot))*6,projectiles[id].y+math.cos(math.rad(rot))*6)
			particlefadealpha(0.04)
		end
	end
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*3,projectiles[id].y-math.cos(math.rad(rot))*3)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				projectiles[id].timer=0
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery() then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			-- End Turn
			endturn()
			break
		end
	end
	-- Timer / Explode
	if (fx==1) then
		projectiles[id].timer=projectiles[id].timer-1
	end
	if projectiles[id].timer<=0 then
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,120,60)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,35,1)
		-- Crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater150,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end