--------------------------------------------------------------------------------
-- Weapon Rat + Projectile Rat
-- Original Carnage Contest Weapon
-- Script by DC, March 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.rat={}
cc.rat.rat={}

-- Load & Prepare Ressources
cc.rat.gfx_wpn0=loadgfx("weapons/rat0.bmp")					-- Weapon Image Frame 0
setmidhandle(cc.rat.gfx_wpn0)
cc.rat.gfx_wpn1=loadgfx("weapons/rat1.bmp")					-- Weapon Image Frame 1
setmidhandle(cc.rat.gfx_wpn1)
cc.rat.sfx_attack=loadsfx("rat.ogg")						-- Attack Sound
cc.rat.sfx_step=loadsfx("ratstep.ogg")						-- Step Sound

--------------------------------------------------------------------------------
-- Weapon: Rat
--------------------------------------------------------------------------------

cc.rat.id=addweapon("cc.rat","Rat Attack",cc.rat.gfx_wpn0,1,2)			-- Add Weapon (1 use, first in round 2)

function cc.rat.draw()													-- Draw
	-- Draw
	if weapon_shots<=0 then
		-- Info
		hudinfo("Hit [Space] once to launch the rat!")
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(getplayerdirection(0),1)
			setrotation(-(20+math.sin(os.clock()*30)*15)*getplayerdirection(0))
			drawimage(cc.rat.gfx_wpn0,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)-1)
		end
	end
end

function cc.rat.attack(attack)									-- Attack
	if (weapon_shots<=0) then
		-- Fire a projectile (on release/full charge)
		if (attack==1) then
			-- No more weapon switching!
			useweapon(0)
			-- No more player control!
			playercontrol(0)
			-- Set Round Time (15 sec)
			secondsleft=math.floor(getframesleft()/50)
			changeturntime(15-secondsleft)
			-- Launch
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.rat.rat.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*8
			projectiles[id].y=getplayery(0)
			if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
				if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
					projectiles[id].x=getplayerx(0)
				end
			end
			projectiles[id].px=projectiles[id].x
			projectiles[id].py=projectiles[id].y
			-- Set Speeds
			projectiles[id].sy=0
			-- Set direction of projectile
			projectiles[id].dir=getplayerdirection(0)
			-- Animation
			projectiles[id].frame=0
			projectiles[id].timer=0
			-- Timer
			weapon_timer=50
			-- Sound
			playsound(cc.rat.sfx_attack)
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Rat
--------------------------------------------------------------------------------

cc.rat.rat.id=addprojectile("cc.rat.rat")		-- Add Projectile

function cc.rat.rat.draw(id)					-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	if projectiles[id].dir==1 then
		setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))+270)
	else
		setscale(1,-1)
		setrotation(math.deg(math.atan2(projectiles[id].x-projectiles[id].px,-(projectiles[id].y-projectiles[id].py)))-90)
	end
	-- Draw projectile
	if projectiles[id].frame==0 then
		drawimage(cc.rat.gfx_wpn0,projectiles[id].x,projectiles[id].y-2)
	else
		drawimage(cc.rat.gfx_wpn1,projectiles[id].x,projectiles[id].y-2)
	end
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	-- Info
	hudinfo("Hit [Space] again to detonate the rat!")
end

function cc.rat.rat.update(id)							-- Update
	-- Save old position (for rotation calculation)
	if getframesleft()%15==0 then
		projectiles[id].px=projectiles[id].x
		projectiles[id].py=projectiles[id].y
	end
	-- Timer
	weapon_timer=weapon_timer-1
	-- Animation
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>7 then
		projectiles[id].timer=0
		projectiles[id].frame=1-projectiles[id].frame
	end
	-- Move Left/Right
	projectiles[id].x=projectiles[id].x+projectiles[id].dir
	if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
		if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
			-- Upwards/Downwards space?
			doturn=1
			for i=1,14,1 do
				if (i%2)==1 then offset=(i+1)/2 else offset=-(i/2) end
				if collision(col5x5,projectiles[id].x,projectiles[id].y+offset)==0 then
					doturn=0; projectiles[id].ignore=0 projectiles[id].y=projectiles[id].y+offset; break
				elseif terraincollision()==0 and objectcollision()==0 and playercollision()==projectiles[id].ignore then
					doturn=0; projectiles[id].y=projectiles[id].y+offset; break
				end
			end
			if doturn==1 then
				-- Move Back / Turn
				projectiles[id].x=projectiles[id].x-projectiles[id].dir
				projectiles[id].dir=projectiles[id].dir*(-1)
				projectiles[id].px=projectiles[id].x
				projectiles[id].py=projectiles[id].y
			end
		end
	else
		projectiles[id].ignore=0
	end
	-- Fall Down
	projectiles[id].sy=projectiles[id].sy+getgravity()*2
	msubt=math.ceil(math.abs(projectiles[id].sy)/3)
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col5x5,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if projectiles[id].timer==0 or projectiles[id].sy>0.5 then
					playsound(cc.rat.sfx_step)
				end
				projectiles[id].sy=0
				projectiles[id].y=projectiles[id].y-msuby
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water (Swim)
		if (projectiles[id].y)>getwatery()-40 then
			projectiles[id].sy=0
			projectiles[id].y=getwatery()-40
			if (getframesleft()%20)==0 then
				playsound(sfx_waterstep)
			end
		end
	end
	-- Timer / Explode
	if getframesleft()<5 or (weapon_timer<=0 and keydown(key_attack)==1) then
		-- Splatter
		playsound(sfx_splatter2)
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,90,50)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,35,1)
		-- Crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		-- Free projectile
		freeprojectile(id)
		-- End Turn
		endturn()
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end