--------------------------------------------------------------------------------
-- Weapon Trap Kit + Projectile Trap Kit
-- Original Carnage Contest Weapon
-- Script by DC, July 2010, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.trapkit={}
cc.trapkit.trapkit={}

-- Load & Prepare Ressources
cc.trapkit.gfx_icon=loadgfx("weapons/trapkit.png")				-- Weapon Icon
setmidhandle(cc.trapkit.gfx_icon)
cc.trapkit.gfx_wpn=loadgfx("weapons/trap.bmp")					-- Weapon
sethandle(cc.trapkit.gfx_wpn,7,9)
cc.trapkit.sfx_attack=loadsfx("throw.ogg")						-- Attack Sound
cc.trapkit.sfx_bounce=loadsfx("bounce.wav")						-- Bounce Sound

--------------------------------------------------------------------------------
-- Weapon: trapkit
--------------------------------------------------------------------------------

cc.trapkit.id=addweapon("cc.trapkit","Trap Kit",cc.trapkit.gfx_icon,0,2)	-- Add Weapon (0 uses, first in round 2)
cc.trapkit.ammo=3															-- 3 Mines

function cc.trapkit.draw()													-- Draw
	if weapon_shots<cc.trapkit.ammo then
		if getplayeraction(0)==0 then
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(1,1)
			setrotation(0)
			drawimage(cc.trapkit.gfx_wpn,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)+2)
		end
		-- HUD ammobar
		if cc.trapkit.ammo-weapon_shots>0 then
			hudammobar(cc.trapkit.ammo-weapon_shots,cc.trapkit.ammo)
		end
	end
end

function cc.trapkit.attack(attack)											-- Attack
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	if (weapon_shots<cc.trapkit.ammo) then
		if attack==1 and weapon_timer<=0 then
			-- No more weapon switching!
			useweapon(0)
			playsound(cc.trapkit.sfx_attack)
			weapon_shots=weapon_shots+1
			id=createprojectile(cc.trapkit.trapkit.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*7
			projectiles[id].y=getplayery(0)+2
			-- Set speed of projectile
			projectiles[id].sx=getplayerdirection(0)*0.5
			projectiles[id].sy=-1
			-- Timer
			weapon_timer=50
			-- End Turn
			if weapon_shots>=cc.trapkit.ammo then
				endturn()
			end
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: trapkit
--------------------------------------------------------------------------------

cc.trapkit.trapkit.id=addprojectile("cc.trapkit.trapkit")		-- Add Projectile

function cc.trapkit.trapkit.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	-- Draw projectile
	drawimage(cc.trapkit.gfx_wpn,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.trapkit.trapkit.update(id)					-- Update
	-- Gravity influence on speed + decrease x speed
	projectiles[id].sx=projectiles[id].sx*0.95
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(cc.trapkit.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.05
				msubx=-msubx*0.05
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(col3x3,projectiles[id].x,projectiles[id].y)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(cc.trapkit.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
				-- Remove / Exchange with real trapkit
				if (math.abs(projectiles[id].sy)<0.3) then
					createobject(o_beartrap,projectiles[id].x,projectiles[id].y)
					freeprojectile(id)
				end
			end
		else
			projectiles[id].ignore=0
		end		
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end
